# ///////////////////////////////////////////////////////////////
# TEST_LABEL - Label Widgets Tests Module
# Project: ezqt_widgets
# ///////////////////////////////////////////////////////////////

"""
Unit tests for label widgets.

This module contains unit tests for all label widgets including
ClickableTagLabel, FramedLabel, HoverLabel, and IndicatorLabel.
"""

from __future__ import annotations
