# ///////////////////////////////////////////////////////////////
# TEST_MISC - Miscellaneous Widgets Tests Module
# Project: ezqt_widgets
# ///////////////////////////////////////////////////////////////

"""
Unit tests for miscellaneous widgets.

This module contains unit tests for all miscellaneous widgets including
CircularTimer, DraggableList, OptionSelector, ToggleIcon, and ToggleSwitch.
"""

from __future__ import annotations
