# ///////////////////////////////////////////////////////////////
# TEST_INPUT - Input Widgets Tests Module
# Project: ezqt_widgets
# ///////////////////////////////////////////////////////////////

"""
Unit tests for input widgets.

This module contains unit tests for all input widgets including
AutoCompleteInput, SearchInput, TabReplaceTextEdit, and PasswordInput.
"""

from __future__ import annotations
